;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "mediawiki" "2.4.10"
  "Mediawiki frontend."
  '((emacs "28.1"))
  :url "https://github.com/hexmode/mediawiki-el"
  :commit "cf091148fd8fcf17d81bc5ad556ae18c839f6507"
  :revdesc "cf091148fd8f"
  :keywords '("mediawiki" "wikipedia" "network" "wiki")
  :authors '(("Mark A. Hershberger" . "mah@everybody.org"))
  :maintainers '(("Mark A. Hershberger" . "mah@everybody.org")))
